<?php
################################################################################
##                    OpenWebStats Version 1.0                                ##
##                                                                            ##
##        (c)2005 mathews_dm - <Davidmathews@open-creations.com>              ##
##                                                                            ##
################################################################################
## Based on  Pascal Toussaint's: <pascal@pascalz.com>                         ##
## CLASS: Apache Logs Reader                                                  ##
################################################################################
## Do Not Edit Below This Line                                                ##
## Please read the README!                                                    ##
## Please edit the config.php file to set the db info and the logfile location##
## Then set a crontab to update the db automatically                          ##
## Example Crontab Entry to update every hour                                 ##
## 59 * * * * php class_apache_logs_import.php >apache_log_import.log 2>&1    ##
## Otherwise you can just click the update db link on the menu of the web to  ##
## run this script via the webpage                                            ##
################################################################################
include('config.php');

set_time_limit (0);
ini_set("memory_limit","12M");

function unquote(&$str)
{
	$str = trim($str);
	
	if (($str[0] == '"')&& ($str[strlen($str) - 1] == '"'))
	{
		$str = substr($str,1,strlen($str) - 2);			
	}
}

$log = new ApacheLogsReader($logfile, $logformat);

class ApacheLogsReader
{
	var $logfile;
	var $_logformat;
	var $_phpformat;
	var $_infos = array();
	
	function ApacheLogsReader($logfile,$logformat = "")
	{
		global $databasename, $openstats_web_dir;
		 $this->logfile = $logfile;
		 $this->_phpformat = "";
		$convert_date = date("m-d-Y H:i:s");
		echo "<b>OpenWebStats version 1.0</b><br>\n";
		echo "Starting db conversion ".$convert_date."<br>\n";
		echo "This can take a while so please stand by.....<br>\n";
		echo "<img src='$openstats_web_dir/images/standby.gif'> <br>";
		flush();
		 if ($logformat != "")
			 $this->SetLogFormat($logformat);
		 else $this->SetLogFormat(logformat_combined);
		$convert_date = date("m-d-Y H:i:s");
		echo "Finished db conversion ".$convert_date."<br>\n";
		echo "OpenWebStats has successfully updated the database and will now exit.<br>\n";
	}
	
	function SetLogFile($logfile)
	{
		$this->logfile = $logfile;	
	}

	function Error($error,$stop = false)
	{
		print($error);
		error_reporting(E_ALL);
		if ($stop) exit;
	}
	
	function SetLogFormat($logformat)
	{
		$this->_logformat = $logformat;
		$this->_LogFormatToPHPFormat($this->_logformat);
		$this->_LoadLogFile();
	}

	function _ReplaceCharInsideQuote($str,$char,$replace)
	{
		$ret = ""; 
		$open = false;
		for ($i = 0; $i < strlen($str); $i++)
		{
			
			if($str{$i} == '"')
			{
				$open = !$open;
				$ret.= $str{$i};
			}elseif($str{$i} == $char)
			{
				if ($open) 
					$ret.= $replace; 
				else $ret.=$str{$i}; 
			}else $ret.= $str{$i};
		}
		return $ret;
	}
	
	function _LogFormatToPHPFormat($logformat)
	{
		$lf_array = explode(" ",$logformat);	
		foreach($lf_array as $value)
		{
			unquote($value);
			
			$elem = substr($value,0,1).substr($value,-1);

			switch ($elem)
			{
				case "%b" :
					$this->_phpformat.="%s ";
					$this->_infos['Octets'] = 0;
					break;
				case "%f" :
					$this->_phpformat.="%s ";
					$this->_infos['filename'] = "";
					break;
				case "%e" :
				    $this->_phpformat.="%s ";
					$this->_infos[substr($value, strpos($value, "{") + 1, strpos($value, "}") - strpos($value, "{") - 1)] = "";
					break;
				case "%h" :
					$this->_phpformat.="%s ";
					$this->_infos['host'] = "";
					break;
				case "%i" :
					$this->_phpformat.="%s ";
					$this->_infos[substr($value, strpos($value, "{") + 1, strpos($value, "}") - strpos($value, "{") - 1)] = "";
					break;
				case "%l" :
					$this->_phpformat.="%s ";
					$this->_infos['identity'] = "";
					break;
				case "%n" :
					$this->_phpformat.="%s ";
					$this->_infos[substr($value, strpos($value, "{") + 1, strpos($value, "}") - strpos($value, "{") - 1)] = "";
					break;
				case "%o" :
					$this->_phpformat.="%s ";
					$this->_infos[substr($value, strpos($value, "{") + 1, strpos($value, "}") - strpos($value, "{") - 1)] = "";
					break;
				case "%p" :
					$this->_phpformat.="%s ";
					$this->_infos['Port'] = 0;
					break;
				case "%P" :
					$this->_phpformat.="%s ";
					$this->_infos['PID'] = 0;
					break;
				case "%r" :
				    	$this->_phpformat.="%s ";
					$this->_infos['Request'] = "";
					break;
				case "%s" :
					$this->_phpformat.="%s ";
					$this->_infos['CodeHTTP'] = "";
					break;
				case "%t" : 
					$this->_phpformat.="%s %s";
					if (strpos($value,"{")) 
					   $this->_infos['DateTimeF'] = substr($value, strpos($value, "{") + 1, strpos($value, "}") - strpos($value, "{") - 1);
					else $this->_infos['DateTime'] = "";
					$this->_infos['GMT'] = "";
					break;
				case "%T" :
					$this->_phpformat.="%s ";
					$this->_infos['ExecutionTime'] = 0;
					break;
				case "%u" :
					$this->_phpformat.="%s ";
					$this->_infos['RemoteUser'] = "";
					break;
				case "%U" :
					$this->_phpformat.="%s ";
					$this->_infos['URL'] = 0;
					break;
				case "%v" :
					$this->_phpformat.="%s ";
					$this->_infos['VirtualHost'] = 0;
					break;
			}
		}
		$this->_phpformat.="\n";
	}

	function _FormatData($key,$data)
	{
		switch ($key)
		{
			case "DateTime" :
				if (substr($data,0,1) == "[")
					 $data = substr($data,1);
				list($d, $M, $y, $h, $m, $s) = sscanf($data, "%2d/%3s/%4d:%2d:%2d:%2d");

				$date['day'] = $d;
				switch($M)
				{
					case 'Jan': $date['month'] = 1;break;
					case 'Feb': $date['month'] = 2;break;
					case 'Mar': $date['month'] = 3;break;
					case 'Apr': $date['month'] = 4;break;
					case 'May': $date['month'] = 5;break;
					case 'Jun': $date['month'] = 6;break;
					case 'Jul': $date['month'] = 7;break;
					case 'Aug': $date['month'] = 8;break;
					case 'Sep': $date['month'] = 9;break;
					case 'Oct': $date['month'] = 10;break;
					case 'Nov': $date['month'] = 11;break;
					case 'Dec': $date['month'] = 12;break;
				}
				$date['year'] = $y;
				$date['hour'] = $h;
				$date['minute'] = $m;
				$date['second'] = $s;
				$data = $date;
				break;
			case "DateTimeF":
				if (substr($data,0,1) == "[")
					 $data = strftime($this->_infos[$key],strtotime(substr($data,1)));
			    else $data = strftime($this->_infos[$key],strtotime($data));
				break;
			case "GMT":
				if (substr($data,-1) == "]")
					 $data = substr($data,0, strlen($data) - 1);	
				break;
			case "Method" :
				if (substr($data,0,1) == "\"")
					 $data = substr($data,1);
				break;
			case "http":
				if (substr($data,-1) == "\"")
					 $data = substr($data,0, strlen($data) - 1);	
				break;
			default:
			    if ((substr($data,0,1) == "\"")	&& (substr($data,-1) == "\""))
					 $data = substr($data,1,strlen($data) - 2);
				$data = str_replace('',' ',$data);
				break;
		}
		return $data;
	}

	function _LoadLogFile()
	{
		global $databasename;

		if (file_exists($this->logfile)) {
			$hFile = fopen($this->logfile,"r");
			if($hFile) {
				$line = fgets($hFile);
				while (!feof($hFile)) 
				{
					$line = $this->_ReplaceCharInsideQuote($line,' ','');
					$line = sscanf($line, $this->_phpformat); 
					$i = 0;
					foreach(array_keys($this->_infos) as $key)
					{
						$inf[$key] = $this->_FormatData($key,$line[$i]);
						if($key == "Request") {
							list($method,$request,$http) = explode(" ",$inf['Request']);
							$inf['Request'] = $request;
							$inf['Method'] = $method;
							$inf['http'] = $http;
						}
						$i++;
					}
					$query = "INSERT into webstats (host,identity,remoteuser,datetime,request,codehttp,octets,referer,useragent,method,http,vhost) VALUES ('".$inf["host"]."','".$inf["identity"]."','".$inf["RemoteUser"]."','".$inf["DateTime"]["year"]."-".$inf["DateTime"]["month"]."-".$inf["DateTime"]["day"]." ".$inf["DateTime"]["hour"].":".$inf["DateTime"]["minute"].":".$inf["DateTime"]["second"]."','".$inf["Request"]."','".$inf["CodeHTTP"]."','".$inf["Octets"]."','".$inf["Referer"]."','".$inf["User-Agent"]."','".$inf["Method"]."','".$inf["http"]."','".$inf["VirtualHost"]."')";
	        			$insert = mysql_db_query($databasename, $query);
					$line = fgets($hFile);
				}
				fclose($hFile);
			} else {
				echo "<div align='center'><b>Error Updating Stats</b></div>";
				echo "This script does not have rights to read the logfile: ".$this->logfile.".<br>Make sure you gave the correct path<br />\n";
           			echo "Please fix the path to the logfile at the top of this script.<br />\n";
           			exit;
			}
		} else {
			echo "<div align='center'><b>Error Updating Stats</b></div>";
			echo "This script can't find the log file to process or does not have rights to read it.<br>Make sure you gave the correct path<br />\n";
           		echo "Please fix the path to the logfile at the top of this script.<br />\n";
           		exit;
		}
	}
}
?>
